#!/usr/bin/perl

use strict;
use Getopt::Long;
use File::Basename;
use Cwd;

my $dirname = Cwd::abs_path(dirname(__FILE__));

###readin the config file 
my $f = $dirname."/diamund.cfg";
my %h;

open (C,"<$f") or die "can't open config file $f ! \nMake sure you have it in the same dir with diamund.pl\n$f should be a tab delimited file with 2 columns: \n\t-first column contains the name of the software(do NOT change the first column!);\n\t-second column is the path to the software (you need to change the path according with where the software from the first colum is located on your machine!)\n";
while (<C>){
  chomp;
  my @a = split(/\s+/, $_);
  print "missing info in $f file : $_ \t $#a \n" if $#a != 1;
  $h{$a[0]} = $a[1];
}
close(C);

my $mummer = $h{"mummer"}; # don't need it if you use kraken
my $vecscreen_data = $h{"vecscreen_data"}; # don't need it if you have vector kmers 
my $kraken = $h{"kraken"}; # alignment program available from http://ccb.jhu.edu/software.shtml
my $human_index = $h{"human_index"}; # bowtie2 index
my $human_fasta = $h{"human_fasta"}; # a fasta file of the human hg19 genome
my $jellyfish = $h{"jellyfish"}; # available from http://www.cbcb.umd.edu/software
my $minimo = $h{"minimo"}; # available from the AMOS software package at http://www.cbcb.umd.edu/software
my $bowtie = $h{"bowtie"};# you also need bowtie2, and samtools installed
my $samtools = $h{"samtools"};
my $bcftools = $h{"bcftools"};
my $vecscreen = $h{"vecscreen"};
### end config data


my $error_rate=0.0125; # snps above below percentage are considered erroneous
my $kerror=0; # assume kmers occuring this many times are almost certainly errors; initially I had kerror=0

# ********************************************************************
# usage: diamund.pl patient.fq|fa parent1.fq|fa <parent2.fq|fa>
# ********************************************************************

# I might want to allow compressed files -> they need to be de-compressed for jellyfish though

# Example of a run:
# /scratch0/igm2/mpertea/exome/FamilyBH1002>../scripts/diamund.pl /scratch0/igm1/salzberg/Exome_compare/FamilyBH1002/DATA/JH1002_1A_stiff.fa /scratch0/igm1/salzberg/Exome_compare/FamilyBH1002/DATA/JH1002_1B_normal.fa -x /scratch0/igm2/mpertea/exome/27mers-exome -v /scratch0/igm1/salzberg/Exome_compare/vector/vector-27mers -t 10 -c -o variants.called >& variants.err
# or 1> variants.called 2>variants.err


my $usage = q/Usage:
 diamund.pl {<patient>.fa|<patient>.fq[.gz|.gz2][,<patient_mates>.fq[.gz|.gz2]]} {<parent1>.fa|<parent1>.fq[.gz|.gz2][,<parent1_mates>.fq[.gz|.gz2]]}  [{<parent2>.fa|<parent2>.fq[.gz|.gz2][,<parent2_mates>.fq[.gz|.gz2]]}] {-e <exon_file>.fa|-x <exon_kmer_file>} [options]
Options:
 -e <exon_file>         fasta file with the sequence of all exons
 -x <exon_kmer_file>    sorted exome kmer file (either the -e or -x options should be present in the input)
 -k|kmer <kmer_size>    kmer size for jellyfish program (default: 27)
 -r|recessive           run program in recessive mode (default: dominant)
 -t <no_of_threads>     number of threads for the jellyfish program
 -v <kmer_vector_file>  file with vector kmers (if available)
 -z <vector_file>       fasta file with vectors
 -f <filter_threshold>  only keep kmers in patient that appear more than the <filter_threshold> times
 -n <read_no>           only keep assemblies that have at least <read_no> assembled reads 
 -d <vecscreen_data>    screen for vector contamination with vescreen using the data from this directory
 -g <[0|1](,[0|1])*>    a string of 0 and 1 separated by commas, and signifying the sex associated with each data sample respectively; 1 is male, and 0 is female  
 -l <read_len>          read length (default:100)
 -o <output_file>       output file with variants called (default is STDOUT)
 -s <sequence_depth>    observed depth of coverage in non-patients in order to call variant (default:10)
 -y <kmer_no>           assume kmers occuring this many times in normals are errors (default: 0)
 -c                     don\'t clean-up the files                  
 -m                     use mummer instead of kraken to align kmers to reads (default: no)
Usage examples:
 diamund.pl patient.fa father.fa mother.fa -e hg19_exome.fa -g 1,0,1
or
 diamund.pl patient.1.fq.gz,patient.2.fq.gz normal.1.fg.gz,normal.2.fg.gz -x 27mers-from-exome-sort
/;

# -p gives the step from where to resume the run

die($usage."Error: at least two input fasta file need to be specified\n") unless @ARGV>=2;

### GLOBAL PARAMETERS

my $cleanup=1;
my $kmer=27;     # kmer size for calling jellyfish
my $recessive=0; # default is dominant detection
my $threads=1;
my $vectorkfile="";
my $vectorfile="";
my $kfilter=10;  # default number of times the unique kmer has to be present in the data
my $exonfile="";
my $exonkfile="";
my $runvecscreen=1;
my $readno=4;   # default minimum number of reads in an assembled contig
my $readlen=100;
my $outfilename="";
my $seqdepth=10;
my $step=0;
my $use_mummer=0;
my $gender="";

### end global parameters

my $dontclean=0;


GetOptions('e=s' => \$exonfile,'g=s' => \$gender,'x=s' => \$exonkfile,'k|kmer=i' => \$kmer,'r|recessive' => \$recessive,'t=i' => \$threads,'z=s' => \$vectorfile,'v=s' => \$vectorkfile,'f=i' => \$kfilter, 'd=s' => \$vecscreen_data, 'n=i' => \$readno,'l=i' => \$readlen,'o=s' =>\$outfilename,'s=i'=>\$seqdepth,'m' => \$use_mummer,'c' => \$dontclean, 'p=i' => \$step,'y=i' => \$kerror);

$cleanup=1-$dontclean;

my @sex;

if($gender) {
    @sex=split(/,/,$gender);
    if($#ARGV > $#sex) {
	die($usage."Sex assignments should be made for all samples!\n");
    }
}

if(!$exonfile && !$exonkfile) { 
    die($usage."Error: exon file not present!\n");
}
elsif(!$exonkfile) {
    my $exonroot=($exonfile =~ /([^\/]+).fa$/);
    if($step<1) { $exonkfile=generate_kmers($exonfile,$exonroot,0);} # step 0
}

if(!(-e $vecscreen_data)) { $runvecscreen=0;}


# *** KMERs GENERATION ****

print STDERR "\n[",get_time(),"]: Generate k-mers\n";

my @f; # $f[0] is the patient file, the rest are parent files

my $fasta=1;
for(my $i=0;$i<@ARGV;$i++) {  
  if(($ARGV[$i] !~ /.fa$/) && ($ARGV[$i] !~ /.fasta$/) && ($ARGV[$i] !~ /.FA$/) && ($ARGV[$i] !~ /.FASTA$/)) { $fasta=0;}
  else { if(!$fasta) { die "All input files should be in the same format!\n";}}
}

for(my $i=0;$i<@ARGV;$i++) {    

  my $file;
  if($fasta) { 
    $file=$ARGV[$i];
    ($f[$i])=($ARGV[$i] =~ /([^\/]+).fa$/);
    if(!$f[$i]) { ($f[$i])=($ARGV[$i] =~ /([^\/]+).fasta$/);}
    if(!$f[$i]) { ($f[$i])=($ARGV[$i] =~ /([^\/]+).FA$/);}
    if(!$f[$i]) { ($f[$i])=($ARGV[$i] =~ /([^\/]+).FASTA$/);}
  }
  else {
    #if($ARGV[$i] =~ /,/) {
    $f[$i]=$i+1;
    $file=$f[$i].".fa";
    paired_to_interleaved($file,$ARGV[$i]);
  }

  if($step<2) { 
      if($i) { generate_kmers($file,$f[$i],0);}
      else { generate_kmers($file,$f[$i],$kfilter);}
  } # step 1
}




# *** FILTER STEP ****

# 0: filter vectors if vector file provided

my $command="";
if(!$vectorkfile || $vectorfile) {
    print STDERR "\n[",get_time(),"]: Vector filtering\n";
}

if(!$vectorkfile && $vectorfile) {
    $vectorkfile=generate_kmers($vectorfile,"vector",0);
}
my $rootinfo=$f[0];
if(-e $vectorkfile) {
    $command="join -v1 $kmer"."mers-".$f[0]." $vectorkfile > $kmer"."mers-".$f[0]."-novec";
    print STDERR "***",get_time(),"Run: $command\n";
    if($step<3) { die "Error: could not run join command\n\t$command\n" if system($command); } # step 2
    $rootinfo.="-novec";
    if($cleanup) { 
	my $toclean=$kmer."mers-".$f[0];
	unlink($toclean);
    }
}


# 1: first filter kmers in patient that are not in exome -> these are normal
print STDERR "\n[",get_time(),"]: Exome kmers filtering\n";

$command="join -v1 $kmer"."mers-".$rootinfo." $exonkfile > $kmer"."mers-".$rootinfo."-noexome";
print STDERR "***",get_time(),"Run: $command\n";
if($step<4) { die "Error: could not run join command\n\t$command\n" if system($command); }  # step 3
if($cleanup) { 
    my $toclean=$kmer."mers-".$rootinfo;
    unlink($toclean);
}
$rootinfo.="-noexome";


# 2: here do the filtering of kmers that appear in patient but not in parents, and occur more than a given threshold
print STDERR "\n[",get_time(),"]: Keep patient kmers only above $kfilter threshold\n";
my @fh;
my $filterfile=$kmer."mers-".$rootinfo;
my @kpos;  # kmers at current positions in files $f[$i] that I am reading
my @kthr;

for(my $i=0;$i<@f;$i++) {
    my $file=$kmer."mers-".$f[$i];   
    if(!$i) { $file=$kmer."mers-".$rootinfo;}
    open($fh[$i],$file) || die("Could not open file $file\n"); 
    if($i) {
	$filterfile.="-not_".$f[$i];
	$rootinfo.="-not_".$f[$i];
    }
    $kpos[$i]="";
    $kthr[$i]=$kerror;
}
$filterfile.=".filter$kfilter";

if($step<5) {

    my $counter=1;

    open(O,">${filterfile}.fa");

    my $handle=$fh[0];
    while(<$handle>) {
	chomp;
	my ($km,$thr)= split;
	my $keep=($thr>$kfilter);
	my $i=1;
	while($keep && $i<@f) {
	    if($kpos[$i] lt $km) {
		($kpos[$i],$kthr[$i])=get_kmer($fh[$i],$kpos[$i],$km,$kthr[$i]);
	    }
	    if(($kpos[$i] eq $km) && ($kthr[$i]>$kerror)) { $keep=0;}
	    $i++;
	}
	if($keep) {
	    print O ">kmer$counter $thr\n$km\n";
	    $counter++;
	}
    }
    close(O);

    print STDERR "There are ",$counter-1," kmers with more than $kfilter occurences that are unique to the patient, and are not in the known exome.\n";
}  # step 4

for(my $i=0;$i<@fh;$i++) { close($fh[$i]);}


# *** RUN MUMMER to find reads that contain the kmers ****
my $readcountfile="reads-with-kmers";
my $file=$f[0].".fa";
if($fasta) { $file=$ARGV[0];}
my $readfafile=$readcountfile.".fa"; # the kmer to read alignment output file
if($step<7) { 
    if($use_mummer) { 
	if($step < 6) { 
	    print STDERR "\n[",get_time(),"]: Run mummer\n";
	    run_mummer($filterfile,$file,$readcountfile);
	} # step 5
	# *** GENERATE FASTA FILE OF READS and RUN VECSCREEN
	print STDERR "\n[",get_time(),"]: Generate fasta file ($readfafile) of reads\n";
	if($step<7) { select_seqs_from_multifasta($file,$readcountfile,$readfafile);} # step 6
    }
    else {
	if($step<6) { 
	    print STDERR "\n[",get_time(),"]: Run kraken\n";
	    run_kraken($filterfile,$file,$readfafile);
	} # step 5
    }
    
} # steps 5 and 6

# *** Assemble the reads with: RUN Minimo
print STDERR "\n[",get_time(),"]: Assemble the reads\n";
my $asmprefix="asm-".$rootinfo."-reads";
my %reads;
run_minimo($readfafile,$asmprefix,\%reads); # step 7
my $contigs_root=$asmprefix."-contigs";

# *** RUN BOWTIE2
print STDERR "\n[",get_time(),"]: Run Bowtie2 and samtools mpileup\n";
# align selected contigs to genome to find which ones align
my $extended_contigs=$asmprefix."-contigs-selected-extended.fa";
my @align; # $align[$n][0] = start position on the genome of extended contig alignment 
           # $align[$n][1] = end position on the genome of extended contig alignment 
           # $align[$n][2] = chromosome of alignment on the genome
my @coord; # $coord[n][m][0]= start position of aligned portion m of contig n
           # $coord[n][m][1]= end position of aligned portion m of contig n
           # $coord[n][m][2]= start position of alignment m from contig n on reference genome
my %name2coord;
align_contigs_to_genome($contigs_root,$extended_contigs,\%reads,\@align,\@coord,\%name2coord); # step 8

$command="${bowtie}-build -q $extended_contigs $asmprefix";
print STDERR "***",get_time(),"Run: $command\n";
if($step<10) { die "Error could not run bowtie2 command\n\t$command\n" if system($command); } # step 9

$command="$samtools faidx $extended_contigs";
print STDERR "***",get_time(),"Run: $command\n";
if($step<11) { die "Error could not run samtools command\n\t$command\n" if system($command); } # step 10

my @b; # resulting bam files
my @s; # resulting output files from smatools mpileup
for(my $i=0;$i<@ARGV;$i++) {
    $b[$i]=align_reads($asmprefix,$ARGV[$i],$f[$i],$fasta,\@align,\%name2coord); # step 11
    $s[$i]=$f[$i]."-to-".$f[0].".bcf";
    $command="$samtools mpileup -BQ0 -m3 -d10000000 -uSDA -f $extended_contigs $b[$i] | $bcftools view -bvcg - > $s[$i]";
    print STDERR "***",get_time(),"Run: $command\n";
    if($step<13) { die "Error could not run samtools command\n\t$command\n" if system($command); } # step 12
}

if($cleanup) {
    my $toclean=$asmprefix."*.bt2";
    unlink($toclean);
}


# *** CALL Variants
print STDERR "\n[",get_time(),"]: Call variants\n";


my %notvalid;
my %depth;
open(F,"$bcftools view $s[0]|") || die "ERROR: Could not open $s[0] to view for bcftools\n";
while(<F>) {
    if(!(/^\#/)) { # line doesn't start with # symbol
	my @dp4=(/DP4=(\d+),(\d+),(\d+),(\d+)/);
	my ($contig,$pos,$id,$alt,$ref,@rest)=split;
	my @r=split(/\,/,$ref);
	if(($alt !~ /N/) && ($r[0] !~ /N/)) {
	    my $mutpos=$contig."\t".$pos."\t".$r[0]."\t".$alt;
	    if($dp4[0]+$dp4[1]<=$kfilter) { 
		$notvalid{$mutpos}++;
	    }
	    else {
		push(@{$depth{$mutpos}},($dp4[0]+$dp4[1],$dp4[2]+$dp4[3]));
	    }
	}
    }
}
close(F);


my %common;
for(my $i=1;$i<@s;$i++) {
    open(F,"$bcftools view $s[$i]|") || die "ERROR: Could not open $s[$i] to view for bcftools\n";
    while(<F>) {
	if(!(/^\#/)) { # line doesn't start with # symbol
	    my ($contig,$pos,$id,$alt,$ref,@rest)=split;
	    my @r=split(/\,/,$ref);
	    my $mutpos=$contig."\t".$pos."\t".$r[0]."\t".$alt;

	    my $isY=0;
	    if($gender) {
		my $c=$name2coord{$contig};
		$isY = ($align[$c][2] =~ /y|Y/);
	    }

	    if($isY && !$sex[$i]) { 
		if(!$notvalid{$mutpos} &&($alt !~ /N/) && ($r[0] !~ /N/) ) {
		    $common{$mutpos}++;
		}
	    }
	    else {

		my ($dp)=(/DP=(\d+)/);
		my @dp4=(/DP4=(\d+),(\d+),(\d+),(\d+)/);

		if($dp>=$seqdepth && ($dp4[0]+$dp4[1])/$dp<$error_rate && ($dp4[0]+$dp4[1]<5) ) {
		    if(!$notvalid{$mutpos} &&($alt !~ /N/) && ($r[0] !~ /N/) ) {
			$common{$mutpos}++;
			if(!$depth{$mutpos}) { push(@{$depth{$mutpos}},(0,0));}
			push(@{$depth{$mutpos}},($dp4[0]+$dp4[1],$dp4[2]+$dp4[3]));
		    }
		}
	    }
	}
    }
    close(F);
}

if($outfilename) {
    open(OUTF,">$outfilename");
    select(OUTF);
}

my %pos;
my $seebam=0;
my $unaligned=1;

print "#CHROM\tPOS\tREF\tALT\tCOMMENT\n";
my %seen;
foreach my $m (keys %common) {
    my ($contig,$cpos,$ref,$alt)=split(/\t/,$m);
    my $c=$name2coord{$contig};
    my $isY = 0;
    if($gender) { $isY = ($align[$c][2] =~ /y|Y/);}

    if($common{$m}==$#s && (!$isY || $sex[0])) { # I saw the muation vs. all non-patients, and if it is on chrY then the patient is male
	if(defined $c) { # contig was aligned to reference
	    my $chr=$align[$c][2];
	    my $rpos=get_coord($cpos,\@{$coord[$c]}); 
	    if($rpos) { # this is a valid position not outside the aligned contig
		my $candidate=$chr."\t".$rpos."\t".$ref."\t".$alt."\t";
		if($depth{$m}[1]) {
		    my $perc=sprintf("%.2f",100*$depth{$m}[0]/($depth{$m}[1]+$depth{$m}[0]));
		    $seen{$candidate}.="Patient coverage = ".$depth{$m}[0]."|".$depth{$m}[1]." (${perc}%);";
		}
		else {
		    push(@{$pos{$contig}},[($cpos,uc($ref),uc($alt),0,0,$candidate)]);
		    $seebam=1;
		}
		for(my $i=0;$i<$#s;$i++) {
		    my $perc=sprintf("%.2f",100*$depth{$m}[2+2*$i]/($depth{$m}[3+2*$i]+$depth{$m}[2+2*$i]));
		    $seen{$candidate}.="Normal[".($i+1)."] coverage = ".$depth{$m}[2+2*$i]."|".$depth{$m}[3+2*$i]."(${perc}%);";
		}
		$seen{$candidate}.="Contig=$contig;Contig position=$cpos;";
	    }
	}
	else { # unaligned contig 
	    my $candidate="Unaligned\t".$unaligned."\t".$ref."\t".$alt."\t";
	    $unaligned++;
	    if($depth{$m}[1]) {
		my $perc=sprintf("%.2f",100*$depth{$m}[0]/($depth{$m}[1]+$depth{$m}[0]));
		$seen{$candidate}.="Patient coverage = ".$depth{$m}[0]."|".$depth{$m}[1]." (${perc}%);";
	    }
	    else {
		push(@{$pos{$contig}},[($cpos,uc($ref),uc($alt),0,0,$candidate)]);
		$seebam=1;
	    }
	    for(my $i=0;$i<$#s;$i++) {
		my $perc=sprintf("%.2f",100*$depth{$m}[2+2*$i]/($depth{$m}[3+2*$i]+$depth{$m}[2+2*$i]));
		$seen{$candidate}.="Normal[".($i+1)."] coverage = ".$depth{$m}[2+2*$i]."|".$depth{$m}[3+2*$i]."(${perc}%);";
	    }
	    $seen{$candidate}.="Contig=$contig;Contig position=$cpos;\n";
	}
    }
}

if($seebam) {

    #print STDERR "Run samtools command: view $b[0]\n";

    open(F,"$samtools view $b[0]|");
    while(<F>) {
	my ($qname,$flag,$rname,$readstart,$mapq,$cigar,$rnext,$pnext,$tlen,$readseq,@rest)=split;
	if(!($flag & 4) && $pos{$rname} ) { # if read is mapped and the contig is of interest to check
	    $readseq=uc($readseq);

	    my ($md)=($_ =~ /MD:Z:((\d+)(([A-Z]|\^[A-Z]+)(\d+))*)/);

	    my @c;
	    my $len=match_read($readstart,\$readseq,$cigar,$md,\@c);
	    
	    if($len) { # read is aligned
		my $readend=$readstart+$len-1;
		
		for(my $j=0;$j<scalar(@{$pos{$rname}});$j++) {
		    my $s=$pos{$rname}[$j][0];

		    if($readstart<=$s && $s<=$readend) {
			my $reflen=length($pos{$rname}[$j][1]);
			my $altlen=length($pos{$rname}[$j][2]);

			my $maxlen=$altlen < $reflen ? $reflen : $altlen;
		    
			my $m=0;
			my $readpos=get_read_coord($s,\@c,\$m);

			#print STDERR "readpos=$readpos for s=$s readstart=$readstart readend=$readend maxlen=$maxlen\n";

			if($readpos && $s<=$readend-$maxlen+1) { # coordinate is on the aligned portion
			    #print STDERR "Found mutation ",substr($readseq,$readpos,$maxlen)," in $rname at pos $s with m=$m: ",$pos{$rname}[$j][1]," -> ",$pos{$rname}[$j][2];
			    if($readpos+$maxlen-1>$c[$m][1]) {
				$pos{$rname}[$j][4]++;
				#print STDERR " that crosses border\n";
			    }
			    else {
				my $base=substr($readseq,$readpos,$altlen);
				if($base eq $pos{$rname}[$j][2]) {
				    $pos{$rname}[$j][3]++;
				}
				else { $pos{$rname}[$j][4]++;}
				#print STDERR " with base=$base\n";
			    }
			}
		    }
		}
	    }
	}
    }
    close(F);

    foreach my $contig (keys %pos) {
	for(my $j=0;$j<scalar(@{$pos{$contig}});$j++) {
	    if($pos{$contig}[$j][3]<=$kfilter) { delete $seen{$pos{$contig}[$j][5]};}
	    else {
		my $perc=sprintf("%.2f",100*$pos{$contig}[$j][3]/($pos{$contig}[$j][3]+$pos{$contig}[$j][4]));
		$seen{$pos{$contig}[$j][5]}="Patient coverage = ".$pos{$contig}[$j][3]."|".$pos{$contig}[$j][4]." (${perc}%);".$seen{$pos{$contig}[$j][5]};
	    }
	}
    }

}

foreach my $m (sort (keys %seen)) {
    print $m,$seen{$m},"\n";
}

if($outfilename) {
    select(STDOUT);
    close(OUTF);
}

#************ Subroutines **************

sub get_read_coord {
    my ($rpos,$coord,$m)=@_;

    $$m=0;    
    my $cpos=0;
    my $n=scalar(@{$coord});
    while($$m<$n && ($rpos > $$coord[$$m][2]+$$coord[$$m][1]-$$coord[$$m][0])) { $$m++;}

    if($$m<$n) { 
	$cpos=$$coord[$$m][0]+$rpos-$$coord[$$m][2];
    }

    return($cpos);
}


sub get_coord {
    my ($cpos,$coord)=@_; # given the coordinate on the contig, find out the coordinate on reference

    my $rpos=0;
    my $m=0;
    my $n=scalar(@{$coord});
    while($m<$n && $cpos>=$$coord[$m][0]) {
	if($cpos<=$$coord[$m][1]) {
	    return($$coord[$m][2]+$cpos-$$coord[$m][0]);
	}
	$m++;
    }

    return($rpos);
}

sub get_kmer {
    my ($fh,$km,$kmerstop,$thr)=@_;
    my $line;
    while(($km lt $kmerstop) &&($line=<$fh>)) { 
	($km,$thr)= split(/\s+/,$line);
    }

    return($km,$thr);
}

sub align_reads {
    my ($asmprefix,$file,$name,$fasta,$align,$name2coord)=@_;

    my $bamfile="B2align-to-${name}";
    if($step>=12) { $bamfile.=".bam";return($bamfile);}
    my $saminitfile="B2align-to-${name}-init.sam";
    my $baminitfile="B2align-to-${name}-init.bam";

    my $tempfqfile="aligned_reads-to-${name}.fq";
    open(T,">$tempfqfile");

    my $command="${bowtie} -t -x $asmprefix -U $file -S $saminitfile -k 1 --no-unal ";
    if($fasta) { $command.="-f";}
    print STDERR "***",get_time(),"Run: $command\n";
    die "Error: could not run bowtie2 command\n\t$command\n" if system($command); 	

    open(F,$saminitfile);
    while(<F>) {
	if($_ !~ /^@/) { # not header
	    my ($qname,$flag,$rname,$readstart,$mapq,$cigar,$rnext,$pnext,$tlen,$readseq,$quals,@rest)=split;
	    print T "\@",$qname,"\n$readseq\n\+\n$quals\n";
	}
    }
    close(F);
    close(T);

    open(O,"|$samtools view -S -b - >$baminitfile");
    open(F,$saminitfile);
    my $nextline="";
    while(<F>) {
	if(/^@/) { # header
	    print O $_;
	}
	else {
	    $nextline=$_;
	    last;
	}
    }

    $command="${bowtie} -t -x $human_index -U $tempfqfile -k 1 --no-head";
    print STDERR "***",get_time(),"Run: $command\n";
    open(B,"$command|");
    while(<B>) {
	my ($qname,$flag,$rname,$readstart,$mapq,$cigar,@rest)=split;
	
	die("Read $qname not in order in sam file\n") unless($nextline =~ /^$qname/);

	if($rname eq "*") { # read was unaligned in the genome
	    print O $nextline;
	}
	else {
	    my ($ashuman)= ($_ =~ /AS:i:(-?\d+)/);
	    my ($ascontig)= ($nextline =~ /AS:i:(-?\d+)/);

	    if($ascontig >= $ashuman) { print O $nextline;} # better score when aligned to contig then reference
	    else { # not better score -> should check if read comes from about the same region
		my ($cqname,$cflag,$crname,@rest)=split(/\t+/,$nextline);

		my $c=$name2coord{$crname};
		if(defined $c) { # contig was mapped to genome -> check if read aligns in the same place of genome as contig
		    my $readalignlen=get_read_alignment_len($cigar);
		    if(($rname eq $$align[$c][2]) && ($readstart+$readalignlen >= $$align[$c][0]) && ($$align[$c][1]>=$readstart)) { # read overlaps contig alignment
			print O $nextline;
		    }
		}
	    }
	}

	$nextline=<F>;
    }
    close(B);
    close(F);
    close(O);

    unlink($tempfqfile);

    $command="$samtools sort $baminitfile $bamfile";
    print STDERR "***",get_time(),"Run: $command\n";
    die "Error: could not run samtools command\n\t$command\n" if system($command); 	

    unlink($saminitfile);
    unlink($baminitfile);

    $bamfile.=".bam";

    return($bamfile);
}

sub overlap {
    my ($s1,$e1,$s2,$e2)=@_;

    if($e1<$s2 || $e2<$s1) {return(0);}

    return(1);
}

sub get_read_alignment_len {
    my ($cigar)=@_;

    my @m=($cigar =~ /(\d+)([A-Z])/g);
    my $len=0; # length of alignment
    for(my $j=0;$j<=$#m;$j+=2) {
	if(($m[$j+1] eq 'M')||($m[$j+1] eq 'D')) {
	    $len+=$m[$j];
	}
    }

    return($len);
}

sub fasta_seq {
    my ($seq)=@_;

    $seq =~ tr/\n//d; # delete enters
    my $faseq="";
    
    while($seq =~ /(\S{1,60})/g) { $faseq.=$1."\n";}

    return($faseq);
}

sub align_contigs_to_genome {
    my ($contigs_root,$extended_contigs,$reads,$align,$coord,$name2coord)=@_;

#    my $indexfile=$human_fasta.".fai";
#    if(!(-e $indexfile)) {
#      $command="$samtools faidx $human_fasta";
#      die "Error: could not index human fasta file: $human_fasta\n" if system($command);
#    }
    
    my $command="${bowtie} -t -x $human_index -f -U ${contigs_root}.fa --local -k 1 -S ${contigs_root}.sam";
    print STDERR "***",get_time(),"Run: $command\n";
    if($step<9) { die "Error: could not run bowtie2 command\n\t$command\n" if system($command); }

    $command="$samtools view -b -S ${contigs_root}.sam|$samtools sort - ${contigs_root}";
    print STDERR "***",get_time(),"Run: $command\n";
    if($step<9) { die "Error: could not run samtools command\n\t$command\n" if system($command); }

    if($cleanup) { 
	my $toclean=${contigs_root}.".sam";
	unlink($toclean);
    }

    $command="$samtools view ${contigs_root}.bam";
    my $fO;

    my @contig;
    my $contigend=0; # further contig end on the genome
    my $lastref="";

    my $n=0;
    if($step<9) { open($fO,">$extended_contigs");}
    open(F,"$command|");
    while(<F>) {
	my ($qname,$flag,$rname,$readstart,$mapq,$cigar,$rnext,$pnext,$tlen,$readseq,@rest)=split;

	if($rname eq "*") { # contig is unaligned
	    if($$reads{$qname}>$readno) { # only if I need to remember this contig
		$$name2coord{$qname}=-1;
		if($step<9) { 
		    $readseq=uc(fasta_seq($readseq));
		    print $fO ">$qname nreads=",$$reads{$qname},"\n$readseq";
		    #print STDERR "Unaligned contig: $qname\n";
		}
	    }
	}
	else {

	    my ($md)=($_ =~ /MD:Z:((\d+)(([A-Z]|\^[A-Z]+)(\d+))*)/);

	    if($md =~ /^(\d+)$/) { # this is a perfect match to the genome -> should be discarded as not relevant
		next;
	    }

	    # merge and print previous contigs if there is no overlap with current one
	    if($contigend && (($rname ne $lastref) || ($readstart>$contigend))) { 
		$n=merge_contigs(\@contig,$fO,$lastref,$n,$align,$coord,$name2coord);
		@contig=();
		$contigend=0;
	    }
	    $lastref=$rname;

	    my $prevseq=$readseq;

	    my @c;
	    my $len=match_read($readstart,\$readseq,$cigar,$md,\@c);
	    if(!$len) { # contig is unaligned
		if($$reads{$qname}>$readno) { # only if I need to remember this contig
		    $$name2coord{$qname}=-1;
		    if($step<9) { 
			$prevseq=uc(fasta_seq($prevseq));
			print $fO ">$qname nreads=",$$reads{$qname},"\n$prevseq";
			#print STDERR "Poorly aligned contig: $qname\n";
		    }
		}
		next;
	    }
	    if($readstart+$len-1>$contigend) { $contigend=$readstart+$len-1;}
	    
	    push(@contig,[($qname,[@c],$readseq,$$reads{$qname})]);
	}
    }
    close(F);

    if($contigend) {
	$n=merge_contigs(\@contig,$fO,$lastref,$n,$align,$coord,$name2coord);
    }

    if($step<9) { close($fO);}
    print STDERR "There are $n contigs that could be aligned to the human genome.\n";
    
    
}

sub match_read {
    my ($readstart,$readseq,$cigar,$md,$c)=@_;

    my @m=($md =~ /(\d+)/g);
    if($#m > 5 ) { return(0);} # poorly align read

    @m=($cigar =~ /(\d+)([A-Z])/g);

    my $len=0; # length of alignment
    my $pos=0;
    my $j=0;
    my $soft=0; # how much clipping is there in the contig
    for(my $j=0;$j<=$#m;$j+=2) {
	if($m[$j+1] eq 'M') {
	    push(@{$c},[$pos,$pos+$m[$j]-1,$len+$readstart]);
	    $len+=$m[$j];
	    $pos+=$m[$j];
	}
	elsif($m[$j+1] eq 'D') {
	    $len+=$m[$j];
	}
	elsif($m[$j+1] eq 'S') {
	    if($j==0) { substr($$readseq,0,$m[$j])=""; $soft+=$m[$j];}
	    else {
		substr($$readseq,-$m[$j])="";$soft+=$m[$j];
	    }
	}
	elsif($m[$j+1] eq 'I') {
	    $pos+=$m[$j];
	}
    }
    
    if($soft>$len/5) { # too much clipping -> consider the contig unaligned
	return(0);
    }
    
    return($len);
}


sub merge_contigs {
    my ($contig,$fO,$lastref,$n,$align,$coord,$name2coord)=@_;

    my $ncontig=scalar(@{$contig});

    #print STDERR "Merge $ncontig contigs on $lastref:";
    #for(my $i=0;$i<$ncontig;$i++) { 
    #print STDERR " ",$$contig[$i][0],"(nreads=",$$contig[$i][3],")";
    #}
    #print STDERR "\n";
    
    if($ncontig==1) { # there are no contigs to merge
	if($$contig[0][3]>=$readno) { 
	    print_contig($fO,$$contig[0],$lastref,$n,$align,$coord,$name2coord);
	    $n++;
	}
	return($n);
    }
    
    my @discard;
    $discard[0]=0;
    
    for(my $i=1;$i<$ncontig;$i++) {
	my $j=0;
	while($j<$i && !merge($$contig[$j],$$contig[$i])) { $j++;}
	if($j<$i) { $discard[$i]=1;}
	else {$discard[$i]=0;}
    }
    
    for(my $i=0;$i<$ncontig;$i++) {
	if(!$discard[$i] && $$contig[$i][3]>=$readno ) {
	    print_contig($fO,$$contig[$i],$lastref,$n,$align,$coord,$name2coord);
	    $n++;
	}
    }

    return($n);
}

sub merge {
    my ($contig1,$contig2)=@_;

    my $nalign=scalar(@{$$contig1[1]});

    my $i=0;
    while($i<$nalign && $$contig2[1][0][2]>$$contig1[1][$i][2]+$$contig1[1][$i][1]-$$contig1[1][$i][0]) {
	$i++;
    }

    if($i==$nalign || ($$contig2[1][0][2] < $$contig1[1][$i][2])) { return(0);}

    my $pos1=$$contig1[1][$i][0]+$$contig2[1][0][2]-$$contig1[1][$i][2];

    my $len1=length($$contig1[2])-$pos1;
    my $len2=length($$contig2[2]);
    my $len = $len1<$len2 ? $len1 : $len2;

    my $seq1=substr($$contig1[2],$pos1,$len);
    my $seq2=substr($$contig2[2],0,$len);
	
    if($seq1 ne $seq2) {return(0);}

    #print STDERR "Merge contigs:",$$contig1[0]," and ",$$contig2[0],"\n";
    
    # here do the actual merging
    # to keep read names:
    if($$contig1[3]==1 && $$contig2[3]==1) { $$contig1[0]="r|r";}
    elsif($$contig2[3]==1) { $$contig1[0].="|r";}
    elsif($$contig1[3]==1) { $$contig1[0]="r|".$$contig2[0];}
    else {$$contig1[0].="|".$$contig2[0];}
    # $$contig1[0].="|".$$contig2[0]; # use this only if I want to keep the reads names too
    $$contig1[3]+=$$contig2[3]; # reads add up to this one
    if($len2>$len1) { # merge coords only if contig2's sequence is longer 

	$$contig1[2].=substr($$contig2[2],$len,$len2-$len); # first update contig1's sequence

	#print STDERR "Contig's seq is now: ",$$contig1[2],"\n";
	
	$i=0;
	$nalign=scalar(@{$$contig2[1]});	
	while($i<$nalign && $len>$$contig2[1][$i][1]) { $i++;}
	if($i<$nalign) { # only here I have some alignment matches
	    #if($$contig1[1][-1][2]+$$contig1[1][-1][1]-$$contig1[1][-1][0]>=$$contig2[1][$i][2]) {
	    if($len>=$$contig2[1][$i][0]) {
		$$contig1[1][-1][1]=$$contig2[1][$i][1]+$pos1;
		$i++;
	    }
	    while($i<$nalign) {
		push(@{$$contig1[1]},[$$contig2[1][$i][0]+$pos1,$$contig2[1][$i][1]+$pos1,$$contig2[1][$i][2]]);
		$i++;
	    }
	}
    }
	    
    return(1);
}



sub run_minimo {
    my ($readfafile,$asmprefix,$reads)=@_;

    my $command="$minimo -D FASTA_EXP=1 -D OUT_PREFIX=$asmprefix $readfafile";
    print STDERR "***",get_time(),"Run: $command\n";
    if($step<8) { die "Error: could not run join command\n\t$command\n" if system($command);  }
    
    my $inputfile=$asmprefix."-contigs.fa";
    open(F,$inputfile);
    while(<F>) {
	if(/^>(\S+)/) {
	    my $contig=$1;
	    my ($nreads) = (/nreads=(\d+)\s+/);
	    if($nreads) { 
		$$reads{$contig}=$nreads;
	    }
	    else {
		$$reads{$contig}=1;
	    }
	}
    }
    close(F);

    my $toclean=$asmprefix."-contigs.qual";
    unlink($toclean);
    if($cleanup) {
	$toclean=$asmprefix."-contigs.afg";    
	unlink($toclean);
    }

}


sub select_seqs_from_multifasta {
    my ($readfile,$readcountfile,$readfafile)=@_;

    open(R,$readfile) || die("could not open file $readfile\n");
    open(C,$readcountfile) || die("could not open file $readcountfile\n");
    open(O,">$readfafile");

    my $count=0;

    my $line=<R>;

    my ($tag) = ($line =~ /^>(\S+)\s+/);
    my $seq="";

    my $line=<C>;
    my ($nextread,$no) = split(/\s+/,$line);
   
    while(<R>) {
	chomp;
	$seq.=$_;
	while(<R>) {
	    if(/^>/) { 
		last;
	    }
	    chomp;
	    $seq.=$_;
	} 

	if($tag eq $nextread) {

	    if($runvecscreen) { # screen read for vector contamination if provided
		#print STDERR "Run command: echo $seq | $vecscreen -d $vecscreen_data -f3\n";
		my $out= `echo $seq | $vecscreen -d $vecscreen_data -f3`;
		if($? == -1 ) { 
		    print STDERR "vecscreen command failed... continuing anyway\n";
		    print O ">$nextread $no\n$seq\n";
		}
		elsif($out =~ /No hits found/) {
		    print O ">$nextread $no\n$seq\n";
		    $count++;
		}
	    }
	    else {
		print O ">$nextread $no\n$seq\n";
		$count++;
	    }

	    $line=<C>;
	    if(!$line) { $tag="";last;}
	    ($nextread,$no) = split(/\s+/,$line);
	}
	($tag) = (/^>(\S+)\s+/);
	$seq="";

    }
    close(R);
    if($tag eq $nextread) {
	if($runvecscreen) { # screen read for vector contamination if provided
	    #print STDERR "Run command: echo $seq | $vecscreen -d $vecscreen_data -f3\n";
	    my $out= `echo $seq | $vecscreen -d $vecscreen_data -f3`;
	    if($? == -1 ) { 
		print STDERR "vecscreen command failed... continuing anyway\n";
		print O ">$nextread $no\n$seq\n";
	    }
	    elsif($out =~ /No hits found/) {
		print O ">$nextread $no\n$seq\n";
		$count++;
	    }
	}
	print O ">$nextread $no\n$seq\n";
	$count++;
    }
    close(O);
    close(C);
    if($cleanup) { unlink($readcountfile);}
    print STDERR "There are $count reads that contain the unique kmers.\n";
}


sub run_mummer {
    my ($filterfile,$readfile,$readcountfile)=@_;
    
    my $mumfile=$filterfile.".mums";
    if(!$cleanup) { open(M,">$mumfile");}
    open(O,">$readcountfile");

    # run mummer
    my $command="$mummer -n -l $kmer -b ${filterfile}.fa $readfile";
    open(F,"$command |");
    # clean up mums; get read counts
    my $count=0;
    my $prev_content="";
    my $prev_read="";

    while(<F>) {
	my ($tag, $readid, $forward) = split(/\s+/);
	if ($tag eq '>') {
	    if($readid ne $prev_read) {
		if($count) { # previous read has content
		    print O $prev_read," ",$count,"\n";
		    if(!$cleanup) { print M $prev_content;}
		}
		$prev_read=$readid;
		$prev_content=$_;
		$count=0;
	    }
	    else {
	    	$prev_content.=$_;
	    }   
	}
	else {
	    $prev_content.=$_;
	    $count++;
	}
    }
    close(F);
    if($count) { # previous read has content
	print O $prev_read," ",$count,"\n";
	if(!$cleanup) { print M $prev_content;}
    }
    close(O);
    if(!$cleanup) { close(M);}
}

sub run_kraken {
    my ($filterfile,$file,$readfafile)=@_;

    my $command="$jellyfish count -m $kmer -s 1000000 -t $threads -C -o kmerdb.jf ${filterfile}.fa";
    print STDERR "***",get_time(),"Run: $command\n";
    die "Error: could not run jellyfish command\n\t$command\n" if system($command); 

    $command="$kraken/db_sort -M -t $threads -n 4 -d kmerdb.jf_0 -o kmerdb.kdb -i kmerdb.idx";
    print STDERR "***",get_time(),"Run: $command\n";
    die "Error: could not run kraken command\n\t$command\n" if system($command); 

    $command="$kraken/classify -d kmerdb.kdb -i kmerdb.idx -o - -t $threads -q -C - $file";
    print STDERR "***",get_time(),"Run: $command\n";
    open(F,"$command |");
    open(O,">$readfafile");
    my $count=0;
    my $name="";
    my $seq="";
    while(<F>) {
	if(/^>(\S+)\s+/) {
	    my $tag=$1;
	    if($name) {
		if($runvecscreen) { # screen read for vector contamination if provided
		    #print STDERR "Run command: echo $seq | vecscreen -d $vecscreen_data -f3\n";
		    my $out= `echo $seq | $vecscreen -d $vecscreen_data -f3`;
		    if($? == -1 ) { 
			#print STDERR "vecscreen command failed... continuing anyway\n";
			print O ">$name\n$seq\n";
			$count++;
		    }
		    elsif($out =~ /No hits found/) {
			print O ">$name\n$seq\n";
			$count++;
		    }
		}
		else {
		    print O ">$name\n$seq\n";
		    $count++;
		}
	    }
	    $name=$tag;
	    $seq="";
	}
	else {
	    chomp;
	    $seq.=$_;
	}
    }
    close(F);
    if($name) {
	if($runvecscreen) { # screen read for vector contamination if provided
	    #print STDERR "Run command: echo $seq | vecscreen -d $vecscreen_data -f3\n";
	    my $out= `echo $seq | $vecscreen -d $vecscreen_data -f3`;
	    if($? == -1 ) { 
		#print STDERR "vecscreen command failed... continuing anyway\n";
		print O ">$name\n$seq\n";
		$count++;
	    }
	    elsif($out =~ /No hits found/) {
		print O ">$name\n$seq\n";
		$count++;
	    }
	}
	else {
	    print O ">$name\n$seq\n";
	    $count++;
	}
    }
    close(O);
    # no need to keep these files ever
    unlink("kmerdb.jf_0");
    unlink("kmerdb.idx");
    unlink("kmerdb.kdb");
    print STDERR "There are $count reads that contain the unique kmers.\n";
}

sub generate_kmers {
    my ($file,$root,$filter)=@_;

    $command="$jellyfish count -o $kmer"."mers-$root".".jf -m $kmer -t $threads -s 3000000000 -c 7 -C $file";
    print STDERR "***",get_time(),"Run: $command\n";
    die "Error: could not run jellyfish command\n\t$command\n" if system($command); 

    my $jf_file="$kmer"."mers-".$root.".jf";
    if(!(-e $jf_file)) {
	$jf_file.="_0";
	if(!(-e $jf_file)) { die("Error: can't find jellyfish file!\n");}
    }

    $command="$jellyfish dump -c $jf_file | perl -ne '(\$k,\$c)=split; if(\$c>$filter) {print \"\$k \$c\\n\";}' | sort -S 100G > $kmer"."mers-".$root;
    print STDERR "***",get_time(),"Run: $command\n";
    die "Error: could not run jellyfish command\n\t$command\n" if system($command); 

    if($cleanup) { unlink($jf_file);}

    return("$kmer"."mers-".$root);
}

sub open_compressed {
    my ($f)=@_;

    my $fh=undef;
    my $fz='';
    if($f =~ m/\.bzi?p?2$/) {
	$fz='bzip2';
    }
    elsif ($f=~m/.g?zi?p?$/) {
	$fz='gzip';
    }
    if($fz) {
	open($fh,$fz." -cd '".$f."'|") || 
	    die("Error creating decompression pipe: $fz -cd '$f' !\n");
    }
    else {
	open($fh,$f) || die("Error opening file $f !\n");
    }
    return($fh);
}

sub paired_to_interleaved {
  my ($file,$pair)=@_;
 
  open(O,">$file");
  
  my @f=split(/,/,$pair);

  my @fh=(undef, undef);
  #determine compression, if any
  my @fz=('','');
  my @foh=(undef, undef);
  for (my $i=0;$i<@f;$i++) {
    last unless $f[$i];
    if ($f[$i]=~m/\.bzi?p?2$/) {
      $fz[$i]='bzip2';
    }
    elsif ($f[$i]=~m/.g?zi?p?$/) {
      $fz[$i]='gzip';
    }
    if ($fz[$i]) {
      open($fh[$i], $fz[$i]." -cd '".$f[$i]."'|") || 
        die("Error creating decompression pipe: $fz[0] -cd '$f[$i]' !\n");
    }
    else {
      open($fh[$i], $f[$i]) || die("Error opening file $f[$i] ! \n");
    }
  }

  my ($ended1, $ended2);
  if($#f<1) { $ended2=1;}
  while (1) {
    my ($rname, $rseq, $rquals);
    unless ($ended1) {
      ($rname, $rseq, $rquals)=getFastq($fh[0]);
      $ended1=1 unless $rname;
    }
    my ($mname, $mseq, $mquals);
    unless ($ended2) {
      ($mname, $mseq, $mquals)=getFastq($fh[1]);
      $ended2=1 unless $mname;
    }
    last if ($ended1 && $ended2);

    print O '>'.$rname."\n$rseq\n" if $rname;
    print O '>'.$mname."\n$mseq\n" if $mname;
  }

  for (my $i=0;$i<@f;$i++) {
    last unless $f[$i];
    close($fh[$i]);
  }

 close(O);
}


sub getFastq {
 my $fh=$_[0]; # file handle
 #parses next FASTQ record
 #returns ($readname, $readseq, $readquals)
 my ($rname, $rseq, $rquals);
 while (<$fh>) {
   ($rname)=(m/^@(\S+)/);
   last if $rname;
   }
 if ($_) {
   while (<$fh>) {
     last if m/^\+/;
     chomp;
     $rseq.=$_;
     }
   if ($_) {
     while (<$fh>) {
       chomp;
       $rquals.=$_;
       last if (length($rseq)<=length($rquals));
       }
   }
  }
 return ($rname, $rseq, $rquals);
}


sub get_time {
    my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
    my @weekDays = qw(Sun Mon Tue Wed Thu Fri Sat Sun);
    my ($second, $minute, $hour, $dayOfMonth, $month, $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime();
    my $year = 1900 + $yearOffset;
    my $theTime = "$hour:$minute:$second, $weekDays[$dayOfWeek] $months[$month] $dayOfMonth, $year";
    return($theTime);
}

sub print_contig {
    my ($fO,$contig,$lastref,$n,$align,$coord,$name2coord)=@_;
    
# contig[0] = contig name
# contig[1] = contig's coords but not clip corrected 
# contig[2] = contig's sequence
# contig[3] = contig's read coverage

    #print STDERR "Print contig ",$$contig[0]," for n=$n\n";


    my $contigname=$$contig[0];
    my $contigstart=$$contig[1][0][2];
    my $len=$$contig[1][-1][2]+$$contig[1][-1][1]-$$contig[1][-1][0]+1-$$contig[1][0][2];

    $$name2coord{$contigname}=$n;
    my $seq="";
    my $clip=$readlen-1;
    if($contigstart-1 <$clip) { 
	$clip=$contigstart-1;
    }
    if($clip) { # there is something that can be clipped from the genome
	#$command="seqmanip -r $lastref".":".($contigstart-$clip)."-".($contigstart-1)." $human_fasta";
      $command="samtools faidx $human_fasta $lastref".":".($contigstart-$clip)."-".($contigstart-1);
	#print STDERR "***Run: $command\n";
	my @out=`$command`;
	for(my $i=1;$i<@out;$i++) { $seq.=$out[$i];}
    }
    $$align[$n][0]=$contigstart-$clip;

    my $ncoord=scalar(@{$$contig[1]});
    for(my $j=0;$j<$ncoord;$j++) { 
	$$coord[$n][$j][0]=$$contig[1][$j][0]+$clip+1;
	$$coord[$n][$j][1]=$$contig[1][$j][1]+$clip+1;
	$$coord[$n][$j][2]=$$contig[1][$j][2];
    }
    
    $seq.=$$contig[2];
		
    #$command="seqmanip -r $lastref".":".($contigstart+$len)."-".($contigstart+$len+$readlen-2)." $human_fasta";
    $command="samtools faidx $human_fasta $lastref".":".($contigstart+$len)."-".($contigstart+$len+$readlen-2);
    #print STDERR "***Run: $command\n";
    my @out=`$command`;
    $clip=0;
    for(my $i=1;$i<@out;$i++) { $seq.=$out[$i];}
    $seq =~ tr/\n//d; # delete enters
    $$align[$n][1]=$$align[$n][0]+length($seq)-length($$contig[2])+$len-1;
    $$align[$n][2]=$lastref;
    if($step<9) { 
	$seq=uc(fasta_seq($seq));
	print $fO ">",$contigname," nreads=",$$contig[3],"\n$seq";
    }
}


